/* src/chanopt.c - Paramtrage des options salons
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: chanopt.c,v 1.39 2004/03/25 20:38:02 romexzf Exp $
 */

#include "main.h"
#include "outils.h"
#include "cs_cmds.h"
#include "fichiers.h"
#include "ban.h"

/*
 * generic_chanopt # OPT Arg..
 */

int generic_chanopt(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *c = parv[1], *cmd = parv[2];
	int i = 0;

	if(!strcasecmp(cmd, "setwelcome"))
		switch_option(nick, parc < 3 ? NULL : parv[3], cmd, c, &chaninfo->flag, C_SETWELCOME);

	else if(!strcasecmp(cmd, "defmodes"))
	{/* nouveau parsage, plus couteux, mais plus fiable -- verif ds le dbut -Cesar*/
		struct cmode tmp = {0};
		char *key, *limit;
		int k = 0, l = 0;

		if(parc < 3)
			return csntc(nick, "Syntaxe: %s #salon DEFMODES <modes>", parv[0]);

		if((key = strchr(parv[3], 'k'))) k = 4; /*+[k!][l?]*/
		if((limit = strchr(parv[3], 'l')) && key > limit) l = k++; /* +[l!][k!]*/
		else if(limit) l = key ? 5 : 4; /*+[k!|?][l!]*/

		if(!string2scmode(&tmp, parv[3], (k && k <= parc) ? parv[k] : NULL, (l && l <= parc) ? parv[l] : NULL))
			return csntc(nick, "Aucun mode valide dfini.");
		if(!IsOper(nick) && tmp.modes & C_MOPERONLY) tmp.modes &= ~C_MOPERONLY;

		memcpy(&chaninfo->defmodes, &tmp, sizeof tmp);

		csmode(chaninfo, 0, "-%s", GetCModes(chaninfo->mode));
		csmode(chaninfo, 0, "+%s", (key = GetCModes(chaninfo->defmodes)));/* hack!*/
		csntc(nick, GetReply(nick, L_DEFMODESARE), c, key);
	}

	else if(!strcasecmp(cmd, "welcome"))
	{
		if(parc < 3)
			return csntc(nick, "Syntaxe: %s #salon WELCOME <message|'none'>", parv[0]);

		if(!strcasecmp(parv[3], "none"))
		{
			*chaninfo->welcome = '\0';
			DelCSetWelcome(chaninfo);
			csntc(nick, GetReply(nick, L_OKDELETED), c);
		}
		else
		{
			parv2msgn(parc, parv, 3, chaninfo->welcome, TOPICLEN);
			csntc(nick, GetReply(nick, L_WELCOMEIS), c, chaninfo->welcome);
		}
	}

	else if(!strcasecmp(cmd, "deftopic"))
	{
		if(parc < 3)
			return csntc(nick, "Syntaxe: %s #salon DEFTOPIC <topic>", parv[0]);

		if(CLockTopic(chaninfo)) return csntc(nick, GetReply(nick, L_LOCKTOPIC), c);

		parv2msgn(parc, parv, 3, chaninfo->deftopic, TOPICLEN);
		csntc(nick, GetReply(nick, L_DEFTOPICIS), c, chaninfo->deftopic);
		cstopic(chaninfo, chaninfo->deftopic);
	}

	else if(!strcasecmp(cmd, "motd"))
	{
		if(parc < 3) return csntc(nick, "Syntaxe: %s #salon MOTD <motd>", parv[0]);

		if(!strcasecmp(parv[3], "none"))
		{
			if(chaninfo->motd) free(chaninfo->motd), chaninfo->motd = NULL;
			csntc(nick, GetReply(nick, L_OKDELETED), c);
		}
		else
		{
			str_dup(&chaninfo->motd, parv2msg(parc, parv, 3, TOPICLEN));
			csntc(nick, GetReply(nick, L_MOTDIS), c, chaninfo->motd);
		}
	}

	else if(!strcasecmp(cmd, "description"))
	{
		if(parc < 3)
			return csntc(nick, "Syntaxe: %s #salon DESCRIPTION <description>", parv[0]);

		parv2msgn(parc, parv, 3, chaninfo->description, DESCRIPTIONLEN);
		csntc(nick, GetReply(nick, L_DESCRIPTIONIS), c, chaninfo->description);
	}

	else if(!strcasecmp(cmd, "chanurl"))
	{
		if(parc < 3)
			return csntc(nick, "Syntaxe: %s #salon CHANURL <url>", parv[0]);

		Strncpy(chaninfo->url, parv[3], 80);
		csntc(nick, GetReply(nick, L_CHANURLIS), c, chaninfo->url);
	}

	else if(!strcasecmp(cmd, "banlevel"))
	{
		int l = ChanLevelbyUserI(nick->user, chaninfo);
		if(parc < 3)
			return csntc(nick, "Syntaxe: %s #salon BANLEVEL <niveau>", parv[0]);

		if(!is_num(parv[3])) return csntc(nick, GetReply(nick, L_VALIDLEVEL));

		if((i = atoi(parv[3])) > l && !IsAnAdmin(nick->user))
			return csntc(nick, GetReply(nick, L_MAXLEVELISYOURS));

		if(chaninfo->banlevel > l && !IsAnAdmin(nick->user))
			return csntc(nick, GetReply(nick, L_ACCESSTOOLOW), c, l, chaninfo->banlevel);

		chaninfo->banlevel = i;
		csntc(nick, GetReply(nick, L_BANLEVELIS), c, chaninfo->banlevel);
	}

	else if(!strcasecmp(cmd, "bantype"))
	{
		if(parc < 3)
			return csntc(nick, "Syntaxe: %s #salon BANTYPE <n du type>", parv[0]);

		if(!is_num(parv[3]) || (i = atoi(parv[3])) > 5 || i < 1)
			return csntc(nick, "Le type de ban doit tre dfini en prcisant un nombre entre 1 et 5.");

		chaninfo->bantype = i;
		csntc(nick, GetReply(nick, L_BANTYPEIS), chaninfo->chan, GetBanType(chaninfo), i);
	}

	else if(!strcasecmp(cmd, "strictop"))
		switch_option(nick, parc < 3 ? NULL : parv[3], cmd, c, &chaninfo->flag, C_STRICTOP);

	else if(!strcasecmp(cmd, "locktopic"))
		switch_option(nick, parc < 3 ? NULL : parv[3], cmd, c, &chaninfo->flag, C_LOCKTOPIC);

	else if(!strcasecmp(cmd, "nobans"))
		switch_option(nick, parc < 3 ? NULL : parv[3], cmd, c, &chaninfo->flag, C_NOBANS);

	else if(!strcasecmp(cmd, "noops"))
		switch_option(nick, parc < 3 ? NULL : parv[3], cmd, c, &chaninfo->flag, C_NOOPS);

	else if(!strcasecmp(cmd, "autoinvite"))
		switch_option(nick, parc < 3 ? NULL : parv[3], cmd, c, &chaninfo->flag, C_AUTOINVITE);

	else if(!strcasecmp(cmd, "chmodeslevel"))
	{
		int l = ChanLevelbyUserI(nick->user, chaninfo);
		if(parc < 3)
			return csntc(nick, "Syntaxe: %s #salon CHMODESLEVEL <niveau>", parv[0]);

		if(!is_num(parv[3])) return csntc(nick, GetReply(nick, L_VALIDLEVEL));

		if((i = atoi(parv[3])) > l && !IsAnAdmin(nick->user))
			return csntc(nick, GetReply(nick, L_MAXLEVELISYOURS));

		if(chaninfo->cml > l && !IsAnAdmin(nick->user))
			return csntc(nick, GetReply(nick, L_ACCESSTOOLOW), c, l, chaninfo->cml);

		chaninfo->cml = i;
		csntc(nick, GetReply(nick, L_CHMODESLEVELIS), c, chaninfo->cml);
	}

	else if(!strcasecmp(cmd, "bantime"))
	{
		time_t timeout = 0;
		if(parc < 3)
			return csntc(nick, "Syntaxe: %s #salon BANTIME <dure>", parv[0]);

		if((timeout = convert_duration(*parv[3] == '%' ? ++parv[3] : parv[3])) < 0)
			return csntc(nick, "Dure incorrecte, le format est <XjXhXm> X tant le nombre respectif d'units, ou 0 pour dsactiver.");

		chaninfo->bantime = timeout;
		if(timeout) csntc(nick, GetReply(nick, L_BANTIMEIS), c, chaninfo->bantime, duration(timeout));
		else csntc(nick, GetReply(nick, L_OPTIONNOWOFF), cmd, c);
	}

	else if(!strcasecmp(cmd, "autolimit"))
	{
		if(parc < 3)
		{
			chaninfo->flag ^= C_FLIMIT;
			if(CFLimit(chaninfo) && (!chaninfo->limit_inc || !chaninfo->limit_min))
			{
				chaninfo->limit_inc = DEFAUT_LIMITINC;
				chaninfo->limit_min = DEFAUT_LIMITMIN;
			}
		}
		else if(!strcasecmp(parv[3], "off")) chaninfo->flag &= ~C_FLIMIT;
		else
		{
			int linc = getoption("-limit_inc", parv, parc, 3, 1);
			int lmin = getoption("-limit_min", parv, parc, 3, 1);

			if((strcasecmp(parv[3], "on") && !lmin && !linc) || lmin < 0 || linc < 0)
				return csntc(nick, "Syntaxe: %s #salon AUTOLIMIT [ON|OFF] [-limit_inc <nombre>] [-limit_min <nombre>]", parv[0]);

			if(!CFLimit(chaninfo)) chaninfo->flag |= C_FLIMIT;
			chaninfo->limit_inc = linc ? linc : (chaninfo->limit_inc ? chaninfo->limit_inc : DEFAUT_LIMITINC);
			chaninfo->limit_min = lmin ? lmin : (chaninfo->limit_min ? chaninfo->limit_min : DEFAUT_LIMITMIN);
		}
		if(CFLimit(chaninfo)) csntc(nick, GetReply(nick, L_AUTOLIMITIS), c, chaninfo->limit_min, chaninfo->limit_inc);
		else csntc(nick, GetReply(nick, L_OPTIONNOWOFF), cmd, c);
	}
	else return csntc(nick, GetReply(nick, L_UNKNOWNOPTION), cmd);
	write_achan(chaninfo);
	return 1;
}

int locktopic(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *pparv[5] = { parv[0], parv[1], "locktopic", parv[2], NULL};

	return generic_chanopt(nick, chaninfo, parc + 1, pparv);
}

int strictop(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *pparv[5] = { parv[0], parv[1], "strictop", parv[2], NULL};

	return generic_chanopt(nick, chaninfo, parc + 1, pparv);
}

int nobans(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *pparv[5] = { parv[0], parv[1], "nobans", parv[2], NULL};

	return generic_chanopt(nick, chaninfo, parc + 1, pparv);
}

int noops(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *pparv[5] = { parv[0], parv[1], "noops", parv[2], NULL};

	return generic_chanopt(nick, chaninfo, parc + 1, pparv);
}

int activwelcome(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *pparv[5] = { parv[0], parv[1], "setwelcome", parv[2], NULL};

	return generic_chanopt(nick, chaninfo, parc + 1, pparv);
}

int deftopic(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *pparv[100];
	int i = 2;

	pparv[0] = parv[0];
	pparv[1] = parv[1];
	pparv[2] = "deftopic";
	for(;i <= parc;i++) pparv[i+1] = parv[i];

	return generic_chanopt(nick, chaninfo, parc + 1, pparv);
}

int defmodes(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *pparv[100];
	int i = 2;

	pparv[0] = parv[0];
	pparv[1] = parv[1];
	pparv[2] = "defmodes";
	for(;i <= parc;i++) pparv[i+1] = parv[i];

	return generic_chanopt(nick, chaninfo, parc + 1, pparv);
}

int description(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *pparv[100];
	int i = 2;

	pparv[0] = parv[0];
	pparv[1] = parv[1];
	pparv[2] = "description";
	for(;i <= parc;i++) pparv[i+1] = parv[i];

	return generic_chanopt(nick, chaninfo, parc + 1, pparv);
}

int csetwelcome(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *pparv[100];
	int i = 2;

	pparv[0] = parv[0];
	pparv[1] = parv[1];
	pparv[2] = "welcome";
	for(;i <= parc;i++) pparv[i+1] = parv[i];

	return generic_chanopt(nick, chaninfo, parc + 1, pparv);
}

int banlevel(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *pparv[5] = { parv[0], parv[1], "banlevel", parv[2], NULL};

	return generic_chanopt(nick, chaninfo, parc + 1, pparv);
}

int bantype(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *pparv[5] = { parv[0], parv[1], "bantype", parv[2], NULL};

	return generic_chanopt(nick, chaninfo, parc + 1, pparv);
}

int define_motd(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *pparv[100];
	int i = 2;

	pparv[0] = parv[0];
	pparv[1] = parv[1];
	pparv[2] = "motd";
	for(;i <= parc;i++) pparv[i+1] = parv[i];

	return generic_chanopt(nick, chaninfo, parc + 1, pparv);
}

int chanurl(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char *pparv[5] = { parv[0], parv[1], "chanurl", parv[2], NULL};

	return generic_chanopt(nick, chaninfo, parc + 1, pparv);
}
