/* src/del_info.c - Suppression d'informations en mmoire
 * Copyright (C) 2002-2004 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IRCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: del_info.c,v 1.56 2004/03/25 20:38:02 romexzf Exp $
 */

#include "main.h"
#include "outils.h"
#include "del_info.h"

int del_abadchan(const char *chaine)
{
	struct badchans *tmp, **tmpp = &badchead;

	for(;(tmp = *tmpp);tmpp = &tmp->next)
		if(!strcasecmp(chaine, tmp->chaine))
		{
			*tmpp = tmp->next;
			free(tmp);
			return 1;
		}
	return 0;
}

void del_access(anUser *user, aChan *chan)
{
	anAccess **acces = &user->accesshead, *tmp;
	aLink *lp, **lpp;

	for(;(tmp = *acces);acces = &tmp->next)
		if(tmp->c == chan)
		{
			*acces = tmp->next;
			break;
		}

	for(lpp = &chan->access;(lp = *lpp);lpp = &lp->next)
		if(lp->value.a->user == user)
		{
			*lpp = lp->next;

			if(tmp->info) free(tmp->info);
			free(tmp);
			free(lp); /* ne pas oublier de free() le link, -memory leak-*/
			return;
		}
	return;
}

void del_killinfo(aNick *nick, const char *against)
{
	aKill **tmpp = &killhead, *tmp;
	/* del de la queue des kills si c'est celui qui pointe vers le nick */
	/* et que nick->nick correspond au 'against', si NULL tout kill est retir*/
	for(;(tmp = *tmpp);tmpp = &tmp->next)
		if(tmp->nick == nick && (!against || !strcasecmp(nick->nick, against)))
		{
			*tmpp = tmp->next;
			free(tmp);
			return;
		}
}

int del_abadnick(const char *chaine)
{
	struct badnicks **bnp = &badnhead, *bn;

	for(;(bn = *bnp);bnp = &bn->next)
		if(strcasecmp(chaine, bn->chaine))
		{
			*bnp = bn->next;
			free(bn);
			return 1;
		}

	return 0;
}

void del_join(aNick *nick, const char *chan)
{
	aJoin **joinp = &nick->joinhead, *join;

	for(;(join = *joinp);joinp = &join->next)
		if(!strcasecmp(chan, join->channel))
		{
			*joinp = join->next;

			if(nick->user && join->chan) /* update le lastseen comme il faut */
			{
				anAccess *acces = GetAccessIbyUserI(nick->user, join->chan);
				if(acces) acces->lastseen = CurrentTS;
			}
			if(join->chan) del_link(join->chan, nick); /* on del le link*/
			free(join);
			return;
		}
}

void del_link(aChan *chan, aNick *nick)
{
	register aLink **lpp = &chan->members, *lp = NULL;

	--chan->users;
	for(;(lp = *lpp);lpp = &lp->next)
		if(lp->value.j->nick == nick)
		{
			*lpp = lp->next;
			if(chan->users == 1) chan->lastact = CurrentTS;
			free(lp);
			return;
		}
}

void del_ban(aChan *chan, aBan *ban)
{
	if(chan->banhead == ban)
	{
		chan->banhead = ban->next;
		if(ban->next) chan->banhead->last = NULL;
	}
	else
	{
		if(ban->last) ban->last->next = ban->next;
		if(ban->next) ban->next->last = ban->last;
	}
	free(ban->raison);
	free(ban);
	return;
}

void del_ignore(const char *host)
{
	struct ignore **ipp = &ignorehead, *ip;

	for(;(ip = *ipp);ipp = &ip->next)
		if(!strcasecmp(host, ip->host))
		{
			*ipp = ip->next;
			free(ip);
			return;
		}
}
