/* src/nickserv.c - Diverses commandes sur le module nickserv
 * Copyright (C) 2002-2004 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: nickserv.c,v 1.147 2004/05/11 18:20:46 romexzf Exp $
 */

#include "main.h"
#include "hash.h"
#include "admin_user.h"
#include "chanserv.h"
#include "outils.h"
#include "cs_cmds.h"
#include "add_info.h"
#include "del_info.h"
#include "config.h"
#include "fichiers.h"
#include "aide.h"
#ifdef HAVE_VOTE
#include "vote.h"
#endif
#ifdef USE_MEMOSERV
#include "memoserv.h"
#endif
#ifdef HAVE_TRACK
#include "track.h"
#endif
#ifdef SQLLOG
#include "sql_log.h"
#endif
#include "nickserv.h"

int register_user(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anUser *user = NULL;
	char p[20];
#ifdef USE_MEMOSERV
	char memo[MEMOLEN + 1];
#endif

	if(nick->user
#ifdef ADMINREG
		&& !IsAnAdmin(nick->user)
#endif
	) return csntc(nick, GetReply(nick, L_LOGUEDIN), nick->user->nick);

	if(NRegister(nick) && !nick->user) /* on check a avant tout parce que a sert  rien de faire les verifs sinon*/
		return csntc(nick, GetReply(nick, L_CMDALREADYUSED), parv[0]);

	if(nick->user || GetConf(CF_PREMIERE|CF_NOMAIL))
	{
		if(parc < 4 || strlen(parv[4]) < 7) return csntc(nick, GetReply(nick, L_PASS_LIMIT));

		Strncpy(p, parv[4], sizeof p -1);
	}
	else Strncpy(p, mkpass(nick->numeric), sizeof p -1);

	if(strlen(parv[1]) > NICKLEN)
		return csntc(nick, GetReply(nick, L_USERLENLIMIT), NICKLEN);

	if(!IsValidNick(parv[1]))
		return csntc(nick, GetReply(nick, L_USER_INVALID), parv[1]);

	if(IsBadNick(parv[1]))
		return csntc(nick, GetReply(nick, L_CANTREG_REASON), parv[1]);
	/* le nick existe mais n'est pas le sien, risque d'abus*/
	if(!nick->user && strcasecmp(nick->nick, parv[1]) && getnickbynick(parv[1]))
		return csntc(nick, GetReply(nick, L_UCANTREG_INUSE));

	if ((user = getuserinfo(parv[1])))
		return csntc(nick, GetReply(nick, L_ALREADYREG), user->nick);

	if(strcasecmp(parv[3], parv[2]))
		return csntc(nick, "Les deux emails fournis sont diffrents, vous avez surement commis une faute de frappe.");

	if(strlen(parv[2]) > MAILLEN)
		return csntc(nick, GetReply(nick, L_MAILLENLIMIT), MAILLEN);

	if(!IsValidMail(parv[2]))
		return csntc(nick, GetReply(nick, L_MAIL_INVALID));

	if(GetUserIbyMail(parv[2])) return csntc(nick, GetReply(nick, L_MAIL_INUSE));

	if(nick->user) user = add_regnick(parv[1], mkpass(p), CurrentTS, 1, U_DEFAULT, parv[2], -1);
	else
	{
		SetNRegister(nick);
		user = add_regnick(parv[1], mkpass(p), CurrentTS, GetConf(CF_PREMIERE) ? 4 : 1,
			GetConf(CF_NOMAIL) ? U_DEFAULT : U_DEFAULT | U_FIRST, parv[2], -1);
	}
	user->lang = DefaultLang;

#ifdef USE_MEMOSERV
	snprintf(memo, MEMOLEN, "Bienvenue sur les services CoderZ. La liste des commandes est disponible via \2/%s %s", cs.nick, RealCmd("showcommands"));
	add_memo(user, "Services", CurrentTS, memo, 0);
#endif

	append_file(USERS_FILE, user->nick);
	write_anuser(user);
#ifdef SQLLOG
 	sql_query("INSERT INTO "SQLUSER" VALUES('%s', UNIX_TIMESTAMP(), 'register', '%s!%s@%s');", user->nick, nick->nick, nick->ident, nick->host);
#endif
	if(nick->user) return csntc(nick, GetReply(nick, L_ADM_USER_REGUED), user->nick, parv[4], user->mail);

	else if(GetConf(CF_PREMIERE|CF_NOMAIL))
	{
		nick->user = user;
		nick->user->n = nick;
		cs_account(nick, user->nick);
		if(GetConf(CF_PREMIERE))
		{
			chaninfo = add_chan(bot.chan, "Salon d'aide  propos des services CoderZ");
			add_access(user, bot.chan, OWNERLEVEL, A_OP|A_VOICE|A_PROTECT, CurrentTS);
			csjoin(chaninfo, 1);
			putlog(LOG_CHANS, "HELLO %s par %s", bot.chan, GetNUHbyNick(nick, 0));
			append_file(CHANS_FILE, chaninfo->chan);
			write_achan(chaninfo);
			write_anuser(user);
			csntc(nick, "Bienvenue sur les Services CoderZ " SPVERSION " !");
			csntc(nick, "Vous avez bien enregistr l'UserName %s avec un accs Administrateur de niveau maximum (\0024\2)", parv[1]);
			csntc(nick, "Le salon %s vient d'tre enregistr sous votre Username", bot.chan);
			ConfFlag &= ~CF_PREMIERE;
		}
		else csntc(nick, GetReply(nick, L_ADM_USER_REGUED), parv[1], parv[3], parv[2]);
		csntc(nick, "\2Vous avez t identifi  TOUS les services");
		csntc(nick, "Lors de votre prochaine connexion vous pourrez vous identifier avec : \2/%s %s %s %s", cs.nick, RealCmd("login"), parv[1], p);
		csntc(nick, "Le mail (%s) que vous avez fourni peut tre utilis pour vous envoyer un pass en cas de problme.", parv[2]);
		csntc(nick, "Vous pouvez voir la liste des commandes avec \2/%s %s\2", cs.nick, RealCmd("showcommands"));
	}
	else
	{
		FILE *fm;
		char mail[250], *login = RealCmd("login"), *set = RealCmd("set");

		if(!(fm = fopen(nick->base64, "w")))
			return csntc(nick, "Impossible d'envoyer le mail, votre pass est %s", p);

		fprintf(fm, "From: \"%s\" <%s@%s>\nTo: \"%s\" <%s>\n", cs.nick, cs.nick, network, user->nick, user->mail);
		fprintf(fm, "Subject: Enregistrement de votre UserName %s - Registering\n\n- English version follows -\n\n", user->nick);
		fprintf(fm, "Ceci est un email automatique du robot %s\n\nVotre mot de passe est : %s \n", cs.nick, p);
		fprintf(fm, "\nVeuillez le conserver.\nPour vous identifier tapez : /%s %s %s %s \n", cs.nick, login, user->nick, p);
		fprintf(fm, "Si vous voulez le changer, tapez: /%s %s PASS <pass>\n\n%s, Services IRC CoderZ de %s\n\n--\n"
				"This is an automatic email from %s (IRC Services of Network %s)\n\n"
				"You have just registered the username '%s' on IRC, the password for it is %s \n\n"
				"In order to identify yourself, type on IRC: /%s %s %s %s \n\nYou can change your password by using /%s %s PASS <pass>\n"
				"You can change the language of the services by typing: /%s %s lang <yourlanguage>\n\n%s, Services IRC CoderZ de %s\n",
				cs.nick, set, cs.nick, network, cs.nick, network, user->nick, p, cs.nick, login, user->nick, p,	cs.nick, set, cs.nick, set, cs.nick, network);
		fclose(fm);
		snprintf(mail, sizeof mail, "%s -t < %s", mailprog, nick->base64);
		system(mail);
		remove(nick->base64);

		csntc(nick, GetReply(nick, L_PASS_SENT), user->mail);
		csntc(nick, GetReply(nick, L_REGISTERTIMEOUT), duration(REGISTER_TIMEOUT));
	}
	return 1;
}

/*
 * oubli_pass <login> <mail>
 */
int oubli_pass(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	FILE *fm;
	anUser *user;
	char passwd[20], mail[200];

	if(GetConf(CF_NOMAIL)) return csntc(nick, GetReply(nick, L_CMDDISABLE));

	if(nick->user) return csntc(nick, GetReply(nick, L_LOGUEDIN), nick->user->nick);

	if(!(user = getuserinfo(parv[1]))) return csntc(nick, GetReply(nick, L_NOSUCHUSER), parv[1]);

	if(UOubli(user)) return csntc(nick, GetReply(nick, L_CMDALREADYUSED), parv[0]);

	if(strcasecmp(user->mail, parv[2]))
		return csntc(nick, GetReply(nick, L_NOTMATCHINGMAIL), user->nick, parv[2]);

	SetUOubli(user), SetNRegister(nick);

	Strncpy(passwd, mkpass(nick->numeric), sizeof passwd-1);
	strcpy(user->passwd, mkpass(passwd));

	if(!(fm = fopen(nick->base64, "w"))) return csntc(nick, "Impossible d'envoyer le mail.");

	fprintf(fm, "From: \"%s\" <%s@%s>\nTo: \"%s\" <%s>\n", cs.nick, cs.nick, network, user->nick, user->mail);
	fprintf(fm, "Subject: Mot de passe oubli\n\n\nVotre mot de passe pour l'username %s est maintenant : %s \n", user->nick, passwd);
	fprintf(fm, "\nVeuillez le conserver.\nPour vous identifier tapez : /%s %s %s %s \n", cs.nick, RealCmd("login"), user->nick, passwd);
	fprintf(fm, "Si vous voulez le changer, tapez: /%s %s pass <pass>\n\n\n%s, Services IRC CoderZ de %s\n", cs.nick, RealCmd("set"), cs.nick, network);
	fclose(fm);
	snprintf(mail, sizeof mail, "%s -t < %s", mailprog, nick->base64);
	system(mail);
	remove(nick->base64);

	csntc(nick, GetReply(nick, L_PASS_SENT), user->mail);
	write_anuser(user);
	return 1;
}

int drop_user(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	char tmp[80] = {0};

	if(parc < 2 || strcasecmp(parv[parc], "confirme"))
		return csntc(nick, GetReply(nick, L_NEEDCONFIRM));

	if(!checkpass(parv[1], nick->user)) return csntc(nick, GetReply(nick, L_BADPASS), nick->user->nick);

	csntc(nick, GetReply(nick, L_OKDELETED), nick->user->nick);

	if(IsAnOwner(nick->user))
		snprintf(tmp, sizeof tmp, "Drop de l'username de l'owner (\2%s\2)", nick->user->nick);

	del_regnick(nick->user, 1, tmp);
	cs_account(nick, NULL);
	return 1;
}

int ns_login(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anUser *user = NULL;
	anAccess *a;
	aJoin *j = NULL;
#ifdef HAVE_TRACK
	struct track *track;
#endif

	if(!(user = getuserinfo(parv[1]))) return csntc(nick, GetReply(nick, L_NOSUCHUSER), parv[1]);

	if(nick->user)
	{
		csntc(nick, GetReply(nick, L_LOGUEDIN), nick->user->nick);
		if(IsAnAdmin(user) && user != nick->user)
		{
			cswall("Admin Login chou sur \2%s\2 par \2%s\2", user->nick, GetNUHbyNick(nick, 0));
			putlog(LOG_FAUTH, "LOGIN %s (dj logu) par %s@%s", user->nick, nick->nick, nick->user->nick);
		}
		return 0;
	}

	if(!checkpass(parv[2], user))
	{
		csntc(nick, GetReply(nick, L_BADPASS), user->nick);
		if(IsAnAdmin(user))
		{
			cswall("Admin Login chou sur \2%s\2 par \2%s\2", user->nick, GetNUHbyNick(nick, 0));
			putlog(LOG_FAUTH, "LOGIN %s (bad password) par %s", user->nick, nick->nick);
		}
		return 0;
	}

	if(user->n && strcasecmp(parv[0], RealCmd("recover")))
	{
		csntc(nick, GetReply(nick, L_ELSEALREADYLOG));
		csntc(user->n, GetReply(user->n, L_ELSETRYTOLOG), nick->nick, RealCmd("set"));
		if(IsAnAdmin(user))
		{
			cswall("Admin Login chou sur \2%s\2 par \2%s\2", user->nick, GetNUHbyNick(nick, 0));
			putlog(LOG_FAUTH, "LOGIN %s (dj utilis) par %s", user->nick, nick->nick);
		}
		return 0;
	}
	if(IsSuspend(user))
	{
		show_ususpend(nick, user);
		if(IsAnAdmin(user))
		{
			cswall("Admin Login chou sur \2%s\2 par \2%s\2", user->nick, GetNUHbyNick(nick, 0));
			putlog(LOG_FAUTH, "LOGIN %s (suspendu) par %s", user->nick, nick->nick);
		}
		return 0;
	}

#ifdef HAVE_TRACK
	if((track = istrack(user)))
		csntc(track->tracker, "[\2Track\2] %s de %s (%s)", parv[0], user->nick, GetNUHbyNick(nick, 0));
#endif

#ifdef USE_NICKSERV
	del_killinfo(nick, user->nick); /* arret du kill, vu que c'est son nick !*/
#endif

	if(UOubli(user)) DelUOubli(user);
	if(UFirst(user)) DelUFirst(user);
	SetNRegister(nick);	/* pour empecher les register si deauth/drop */
	nick->user = user;
	nick->user->n = nick;
	nick->user->lastseen = CurrentTS;

	cs_account(nick, user->nick);

	csntc(nick, GetReply(nick, L_LOGINSUCCESS), user->nick);
#ifdef SQLLOG
 	sql_query("INSERT INTO "SQLUSER" VALUES('%s', UNIX_TIMESTAMP(), '%s', '%s!%s@%s');", user->nick, parv[0],
 		nick->nick, nick->ident, nick->host);
#endif

	if(IsAnAdmin(user))
	{
		csntc(nick, GetReply(nick, L_LOGINADMIN), user->level);
#ifdef USE_WELCOMESERV
		if(*admin_motd) csntc(nick, GetReply(nick, L_LOGINMOTD), admin_motd);
#endif
	}
#ifdef USE_WELCOMESERV
	else if(*user_motd) csntc(nick, GetReply(nick, L_LOGINMOTD), user_motd);
#endif

	if(GetConf(CF_HOSTHIDING) && GetConf(CF_XMODE) && UWantX(user) && !(nick->flag & N_CRYPT))
		putserv("%s " TOKEN_SVSMODE" %s :+x", cs.num, nick->numeric);

#ifdef USE_MEMOSERV
	show_notes(nick);
#endif

#ifdef HAVE_VOTE
	if(Vote.actif && Vote.nbprop && !user->vote && user->level >= Vote.level)
	{
		show_vote(nick);
		csntc(nick, "Pour voter tapez \2/%s %s <n de la proposition>\2.", cs.nick, RealCmd("voter"));
	}
#endif

	/*auto-autoop propos par BugMaster :)*/
	for(a = user->accesshead;a;a = a->next)
	{
		if(AWait(a) || IsSuspend(a->c)) continue;
		if((j = getjoininfo(nick, a->c->chan))) a->lastseen = 1; /* et non pas time(NULL) car il est sur le salon */

		if(j && ((AOp(a) && !IsOp(j)) || (AVoice(a) && !IsVoice(j))) && !ASuspend(a))
		{
			if(AOp(a) && !CNoOps(a->c)) DoOp(j);
			if(AVoice(a)) DoVoice(j);
			csmode(a->c, MODE_OBV, "+%s%s %s %s",IsOp(j) ? "o" : "", IsVoice(j) ? "v" : "",
				IsOp(j) ? nick->numeric : "", IsVoice(j) ? nick->numeric : "");
		}
		if(a->c->motd) csntc(nick, "\2 [Message du jour] - %s -\2 %s", a->c->chan, a->c->motd);
		if(!j && CAutoInvite(a->c) && (HasMode(a->c, C_MINV | C_MKEY | C_MUSERONLY) || (HasMode(a->c, C_MLIMIT) && a->c->users >= a->c->mode.limit)))
			putserv("%s " TOKEN_INVITE " %s :%s", cs.num, nick->nick, a->c->chan);
	}

	write_anuser(user); /* **joli** oubli qui a cout le deluser  beaucoup.. */
	return 1;
}

#ifdef USE_NICKSERV
int recover(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aNick *who, *whoauth = NULL, *whon = NULL;

	if(!nick->user)
	{
		anUser *u;
		if(parc < 2) return syntax_cmd(nick, FindCoreCommand("recover"));
		/* cache if anyone is already logued into this account */
		if((u = getuserinfo(parv[1])) && u->n != nick) whoauth = u->n;
		/* then perform standard login */
		if(!ns_login(nick, chaninfo, parc, parv)) return 0;
		/* now check if anyone else is using my account as nick */
		if((who = getnickbynick(nick->user->nick)) != nick) whon = who;
	}
	else
	{
		if(parc && strcasecmp(nick->user->nick, parv[1])) /* want different nick from his login */
			return csntc(nick, GetReply(nick, L_LOGUEDIN), nick->user->nick);

		if(!strcasecmp(nick->nick, nick->user->nick)) /* already his nick */
			return csntc(nick, GetReply(nick, L_ALREADYYOURNICK), nick->nick);

		if(!(whon = getnickbynick(nick->user->nick))) /* noone use it, use /NICK boring guy!*/
			return csntc(nick, GetReply(nick, L_NICKAVAIABLE), nick->user->nick);
	}

	if(whoauth && (whoauth != whon || !UPKill(nick->user)))
	{ /* personne authe sur le mme username, deauthons le. (sauf si on va le killer LUI)*/
		csntc(whoauth, GetReply(nick, L_DEAUTHBYRECOVER), nick->nick);
		cs_account(whoauth, NULL);/* attention au cas whoauth!=NULL && whon!=NULL && whon!=whoauth*/
	}

	if(whon) /* personne possdant le nick == username */
	{
		if(!UPKill(nick->user))
		{
			char buf[NICKLEN + 1];/* generates a new nick based on current one and appends */
			do { /* a number of at max 4 digits */
				sprintf(buf, "%.20s%d", whon->nick, rand() & 1023);
			} while(getnickbynick(buf));
			putserv("%s " TOKEN_SVSNICK " %s :%s", bot.servnum, whon->numeric, buf);
		}
		else
		{
			if(whoauth == whon) whon->user = NULL;
			putserv("%s " TOKEN_KILL " %s :%s (Pseudo enregistr)", cs.num, whon->numeric, cs.nick);
			del_nickinfo(whon->numeric, "recover");
		}
		csntc(nick, GetReply(nick, L_NICKAVAIABLE), nick->user->nick);
	}
	return 1;
}
#endif

int myaccess(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	int chan = 0, i = 0;
	anAccess *a;

	if(!nick->user) return 0;

	if(parc)
	{
		const char *cmd = parv[1];
		if(!strcasecmp(cmd, "accept"))
		{
			if(parc < 2) return csntc(nick, "Syntaxe: ACCEPT <#salon>");

			if(!(chaninfo = getchaninfo(parv[2]))) return csntc(nick, GetReply(nick, L_NOSUCHCHAN), parv[2]);

			for(a = nick->user->accesshead;a && a->c != chaninfo;a = a->next);

			if(!a || !AWait(a))
				return csntc(nick, GetReply(nick, L_YOUNOPROPACCESS), parv[2]);

			csntc(nick, GetReply(nick, L_NOWACCESS), a->c->chan);
			a->flag &= ~A_WAITACCESS;
			write_anuser(nick->user);
			return 1;
		}
		if(!strcasecmp(cmd, "refuse"))
		{
			if(parc < 2) return csntc(nick, "Syntaxe: REFUSE <#salon>");

			if(!(chaninfo = getchaninfo(parv[2]))) return csntc(nick, GetReply(nick, L_NOSUCHCHAN), parv[2]);

			for(a = nick->user->accesshead;a && a->c != chaninfo;a = a->next);

			if(!a || !AWait(a))
				return csntc(nick, GetReply(nick, L_YOUNOPROPACCESS), parv[2]);

			csntc(nick, GetReply(nick, L_PROPREFUSED), a->c->chan);
			del_access(nick->user, a->c);
			write_anuser(nick->user);
			return 1;
		}
		if(!strcasecmp(cmd, "drop"))
		{
			if(parc < 2) return csntc(nick, "Syntaxe: DROP <#salon>");

			if(!(a = GetAccessIbyUserI(nick->user, getchaninfo(parv[2]))))
				return csntc(nick, GetReply(nick, L_YOUNOACCESSON), parv[2]);

			if(a->level == OWNERLEVEL)
				return csntc(nick, GetReply(nick, L_OWNERMUSTUNREG), parv[2]);

			del_access(nick->user, a->c);
			write_anuser(nick->user);
			csntc(nick, GetReply(nick, L_OKDELETED), parv[2]);
			return 1;
		}
	}

	if(!getoption("-access", parv, parc, 1, -1))
		return show_userinfo(nick, nick->user, 0);

	chan = getoption("-chan", parv, parc, 1, 0);

	for(a = nick->user->accesshead;a;++i, a = a->next)
	{
		if(AWait(a) || (chan && match(parv[chan], a->c->chan))) continue;
		csntc(nick, "- %s -", a->c->chan);
		show_accessn(a, nick->user, nick);
	}

	return csntc(nick, GetReply(nick, L_TOTALFOUND), i, PLUR(i), PLUR(i));
}

int deauth(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
   if(!nick->user) return csntc(nick, "Vous n'tes pas logu.");

   csntc(nick, "Vous venez de vous dloguer de l'UserName \2%s\2", nick->user->nick);
   cs_account(nick, NULL);
   return 1;
}

int user_set(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *arg = parv[1];

	if(!strcasecmp(arg, "mail"))
	{
		if(parc < 2)
			return csntc(nick, "Syntaxe: %s MAIL <nouveau mail>", parv[0]);

		if(strlen(parv[2]) > MAILLEN)
			return csntc(nick, GetReply(nick, L_MAILLENLIMIT), MAILLEN);

		if(!IsValidMail(parv[2]))
			return csntc(nick, GetReply(nick, L_MAIL_INVALID));

		if(!strcasecmp(nick->user->mail, parv[2]))
			return csntc(nick,  GetReply(nick, L_IDENTICMAIL), parv[2]);

		if(GetUserIbyMail(parv[2])) return csntc(nick, GetReply(nick, L_MAIL_INUSE));

		strcpy(nick->user->mail, parv[2]);
		csntc(nick, GetReply(nick, L_OKCHANGED));
	}
	else if(!strcasecmp(arg, "virtualhost"))
	{
		if(!GetConf(CF_XMODE)) return csntc(nick, GetReply(nick, L_CMDDISABLE));
		switch_option(nick, parc < 2 ? NULL : parv[2], arg, nick->user->nick, &nick->user->flag, U_WANTX);
	}
#ifdef USE_NICKSERV
	else if(!strcasecmp(arg, "protect"))
	{
		int protect;
		if(parc < 2 || !is_num(parv[2]) || (protect = atoi(parv[2])) < 0 || protect > 2)
			return csntc(nick, "Syntaxe: %s PROTECT <0/1/2>", parv[0]);

		if(protect == 1)
		{
			SetUPNick(nick->user);
			if(UPKill(nick->user)) DelUPKill(nick->user);
			csntc(nick, "La protection est active: tout user prenant votre username comme pseudo aura son pseudo chang au bout de %d secondes", kill_interval);
		}
		else if(protect == 2)
		{
			SetUPKill(nick->user);
			if(UPNick(nick->user)) DelUPNick(nick->user);
			csntc(nick, "La protection est active: tout user prennant votre username comme pseudo sera kill au bout de %d secondes", kill_interval);
		}
		else
		{
			if(UPNick(nick->user)) DelUPNick(nick->user);
			if(UPKill(nick->user)) DelUPKill(nick->user);
			csntc(nick, "La protection est desactive");
		}
	}
#endif
#ifdef USE_MEMOSERV
	else if(!strcasecmp(arg, "nomemo"))
		switch_option(nick, parc < 2 ? NULL : parv[2], arg, nick->user->nick, &nick->user->flag, U_NOMEMO);
#endif
	else if(!strcasecmp(arg, "pass"))
	{
		if(parc < 2 || strlen(parv[2]) < 7)
			return csntc(nick, GetReply(nick, L_PASS_LIMIT));

		strcpy(nick->user->passwd, mkpass(parv[2]));
		csntc(nick, GetReply(nick, L_OKCHANGED));
	}
	else if(!strcasecmp(arg, "username"))
	{
		if(parc < 2)
			return csntc(nick, "Syntaxe: %s USERNAME <nouvel username>", parv[0]);

		if(nick->user->flag & U_ALREADYCHANGE)
			return csntc(nick, GetReply(nick, L_CMDALREADYUSED), "UserName");

		if(strlen(parv[2]) > NICKLEN)
			return csntc(nick, GetReply(nick, L_USERLENLIMIT), NICKLEN);

		if(!IsValidNick(parv[2]))
			return csntc(nick, GetReply(nick, L_USER_INVALID), parv[2]);

		if(IsBadNick(parv[2]))
			return csntc(nick, GetReply(nick, L_CANTREG_REASON), parv[2]);

		if(getuserinfo(parv[2]))
			return csntc(nick, GetReply(nick, L_ALREADYREG), parv[2]);

		switch_user(nick->user, parv[2]);
		nick->user->flag |= U_ALREADYCHANGE;
		csntc(nick, GetReply(nick, L_OKCHANGED));
	}
	else if(!strcasecmp(arg, "access"))
	{
		int i = 0;
		anAccess *a, *at;

		if(parc < 2 || !is_num(parv[2]) || (i = atoi(parv[2])) < 0 || i > 2)
		{
			csntc(nick, "Syntaxe: %s ACCESS <0/1/2>", parv[0]);
			csntc(nick, "Syntaxe: 0 Refuse toute proposition d'accs");
			csntc(nick, "Syntaxe: 1 Requiert la confirmation/refus");
			csntc(nick, "Syntaxe: 2 Accepte toute proposition d'accs");
			return 0;
		}

		nick->user->flag &= ~(U_POKACCESS | U_PACCEPT | U_PREJECT);
		switch(i)
		{
			case 0:
			nick->user->flag |= U_PREJECT;
			csntc(nick, GetReply(nick, L_PREFUSE));
			for(a = nick->user->accesshead;a;a = at)
			{
				at = a->next;
				if(AWait(a)) del_access(nick->user, a->c);
			}
			break;

			case 1:
			nick->user->flag |= U_POKACCESS;
			csntc(nick, GetReply(nick, L_PASK), cs.nick, RealCmd("myaccess"));
			break;

			case 2:
			nick->user->flag |= U_PACCEPT;
			csntc(nick, GetReply(nick, L_PACCEPT));
			for(a = nick->user->accesshead;a;a = a->next)
				if(AWait(a)) a->flag &= ~A_WAITACCESS;
			break;
		}

	}
	else if(!strcasecmp(arg, "lang"))
	{
		Lang *lang;

		if(parc < 2)
		{
			csntc(nick, GetReply(nick, L_LANGLIST));
			for(lang = DefaultLang;lang;lang = lang->next) csntc(nick, "%s", lang->langue);
			return 0;
		}

		if(!(lang = lang_isloaded(parv[2]))) return csntc(nick, GetReply(nick, L_NOSUCHLANG), parv[2]);

		nick->user->lang = lang;
		csntc(nick, GetReply(nick, L_NEWLANG));
	}
	else if(!strcasecmp(arg, "busy") && IsAnAdmin(nick->user))
		return switch_option(nick, parc < 2 ? NULL : parv[2], arg, nick->user->nick, &nick->user->flag, U_ADMBUSY);
	else return csntc(nick, GetReply(nick, L_UNKNOWNOPTION), arg);

	write_anuser(nick->user);
	return 1;
}
