/* src/timers.c - fonctions de controle appelles periodiquement
 * Copyright (C) 2002-2004 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: timers.c,v 1.45 2004/04/30 13:18:43 romexzf Exp $
 */

#include "main.h"
#include "config.h"
#include "outils.h"
#include "hash.h"
#include "del_info.h"
#include "fichiers.h"
#include "cs_cmds.h"

void check_accounts(void)
{
	anUser *user, *u2;
	int i = 0, deleted = 0;

	for(;i < USERHASHSIZE;++i) for(user = user_tab[i];user;user = u2)
	{
		u2 = user->next;
		if((user->lastseen + maxlastseen) < CurrentTS && !IsAnAdmin(user) && !UNopurge(user))
			cswallops("Purge de l'username %s (Lastseen %s)", user->nick, duration(CurrentTS - user->lastseen));
		else if(UFirst(user) && (user->lastseen + REGISTER_TIMEOUT) < CurrentTS)
			cswallops("Purge de l'username %s par register timeout", user->nick);
		else
		{
			if(user->n) user->lastseen = CurrentTS;
			if(user->suspend && (user->suspend->debut + user->suspend->expire) < CurrentTS && user->suspend->expire > 0)
				user->suspend->expire = -1;
			if(user->cantregchan > 0 && user->cantregchan < CurrentTS) user->cantregchan = -1;
			continue; /* user is ok, go on without purging it. */
		}
		++deleted;
		del_regnick(user, 0, "Purge de l'username de l'owner");
	}
	if(deleted) write_userlist();
}

void check_kills(void)
{
	aKill *tmp = killhead, *next;
	char buf[NICKLEN + 1];

	for(;tmp;tmp = next)
	{
		next = tmp->next;
		if(CurrentTS >= (tmp->timestamp + kill_interval))
		{	/* check if he still deserves its kill */
			if(!strcasecmp(tmp->nick->nick, tmp->against))
			{
				if(TIMER_CHNICK == tmp->type)
				{	/* generates a new nick based on current one and appends */
					do { /* a number of at max 4 digits */
						sprintf(buf, "%.20s%d", tmp->nick->nick, rand() & 1023);
					} while(getnickbynick(buf));

					putserv("%s "TOKEN_SVSNICK" %s :%s", bot.servnum, tmp->nick->numeric, buf);
				}
				else
				{
					putserv("%s "TOKEN_KILL" %s :%s (%s)", cs.num, tmp->nick->numeric, cs.nick,
						(TIMER_BADNICK == tmp->type) ? "Pseudo illgal" : "Pseudo enregistr");
					del_nickinfo(tmp->nick->numeric, "timer kill");
					continue; /* don't try do remove kill from list, del_nickinfo already did it */
				}
			}
			del_killinfo(tmp->nick, NULL);
		}
	}
}

int check_chan(aChan *chan)
{
	int etat = 0; /*1=pas d'owner, 2=pas d'access, 3=lastseen>maxlastseen*/

	if(!chan->access) etat = 2;
	else if(!chan->owner) etat = 1;/* pas d'owner */
	else if(chan->owner->lastseen > 1 && (chan->owner->lastseen + maxlastseen) < CurrentTS
		&& !UNopurge(chan->owner->user)) etat = 3;

	if(chan->suspend && (chan->suspend->debut + chan->suspend->expire) < CurrentTS && chan->suspend->expire > 0)
	{
		chan->suspend->expire = -1;
		if(!CJoined(chan)) csjoin(chan, 0);
		putlog(LOG_CHANS, "SUSPEND OFF (Auto) %s par %s", chan->chan, chan->suspend->from);
	}

	if(etat == 3 || CWantUnreg(chan))/* abandon de l'owner ou salon dj remarqu */
	{
		char raison[300];
		if(etat == 3) snprintf(raison, sizeof raison, "Pas de visite de l'owner depuis \2%s", get_time(chan->owner->lastseen));
		else strcpy(raison, etat == 1 ? "Aucun owner" : "Aucun accs");
		del_chan(chan, raison);
		return 1;
	}
	else if(etat)
	{
		if(etat == 1) cswallops("Le salon %s n'a pas d'owner, il sera supprim  la prochaine purge.", chan->chan);
		else if(etat == 2) cswallops("Le salon %s n'a pas d'access, il sera supprim  la prochaine purge.", chan->chan);
		SetCWantUnreg(chan); /* mark it, *only* warning */
	}
	return 0;
}

void check_bans(int checkchan)
{	/* if checkchan is not null, chan is checked for purge */
	aChan *c, *next;
	int i = 0;
	aBan *b, *b2;

	for(;i < CHANHASHSIZE;++i) for(c = chan_tab[i];c;c = next)
	{
		next = c->next;
		if(checkchan && check_chan(c)) continue;/* 'c' just got purged */
		if(!IsSuspend(c) && CJoined(c))
		{
			int have = 0;

			if(c->users == 1 && !HasMode(c, C_MKEY|C_MINV) && CurrentTS - c->lastact > chanmaxidle)
			{
				cspart(c, "Idle");
				continue;
			}
			if(CFLimit(c)) /* floating limit */
			{
				int newlimit = c->users + c->limit_inc, dif = newlimit - c->mode.limit;
				if(dif < 0) dif = -dif;
				/* only updates the limit if it is worst (dif >= grace) */
				if(c->mode.limit != newlimit && dif >= c->limit_min)
				{
					c->mode.limit = newlimit;
					if(!HasMode(c, C_MLIMIT)) c->mode.modes |= C_MLIMIT;
					putserv("%s "TOKEN_MODE" %s +l %d", cs.num, c->chan, newlimit);
				}
			}
			/* banlist is now expiration-TS-sorted */
			for(b = c->banhead;b && b->fin && b->fin <= CurrentTS;b = b2)
			{
				b2 = b->next;
				csmode(c, MODE_OBV, "-b %s", b->mask);
				del_ban(c, b);
				have = 1;
			}
			if(have) write_bans(c);
		}
	}/* for() */
	return;
}
